package org.jeecg.modules.mq;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.rabbitmq.client.Channel;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.boot.starter.rabbitmq.core.BaseRabbiMqHandler;
import org.jeecg.boot.starter.rabbitmq.listenter.MqListener;
import org.jeecg.common.annotation.RabbitComponent;
import org.jeecg.modules.adweb.product.dto.ProductDTO;
import org.jeecg.modules.adweb.product.service.IAdwebProductService;
import org.jeecg.modules.adweb.site.entity.AdwebSite;
import org.jeecg.modules.adweb.site.service.IAdwebSiteService;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.support.AmqpHeaders;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.messaging.handler.annotation.Header;

import java.util.List;

/**
 * Rabbit MQ - 产品消息接收
 *
 * @author chenpeiqing
 */
@Slf4j
@RabbitComponent(value = "productReceiver")
public class ProductReceiver extends BaseRabbiMqHandler<ProductDTO> {

    @Autowired
    private IAdwebSiteService adwebSiteService;

    @Autowired
    private IAdwebProductService adwebProductService;

    @RabbitHandler
    // ackMode需要设置为MANUAL - BaseRabbiMqHandler.onMessage()有channel.basicAck()操作
    @RabbitListener(queues = "site_product", ackMode = "MANUAL")
    public void onMessage(
            ProductDTO productDTO,
            Channel channel,
            @Header(AmqpHeaders.DELIVERY_TAG) long deliveryTag) {
        super.onMessage(
                productDTO,
                deliveryTag,
                channel,
                new MqListener<ProductDTO>() {
                    @Override
                    public void handler(ProductDTO productDTO, Channel channel) {
                        // 业务处理
                        log.info("收到MQ消息 {}", productDTO);

                        // TODO 产品消息内容唯一性判断

                        // 查看发布当前消息的站点是否是 adweb3 平台中绑定的是否是有效站点, 如果不是则直接确认消息，不做业务处理
                        List<AdwebSite> adwebSites =
                                adwebSiteService.list(
                                        new LambdaQueryWrapper<AdwebSite>()
                                                .like(AdwebSite::getDomain, productDTO.getSiteHost())
                                                .ne(AdwebSite::getStatus, 0));

                        if (adwebSites.isEmpty()) {
                            log.error(
                                    "站点：{} 不是adweb3的有效绑定站点,MQ消息确认失败",
                                    productDTO.getSiteHost());
                            return;
                        }


                        try {
                            // 产品入库
                            adwebProductService.addProduct(
                                    productDTO, adwebSites.get(0));
                        } catch (Exception e) {
                            log.error("询盘入库异常：{}", e.getMessage());
                        }
                    }
                });
    }
}
